==> calligra_2.8.1-1.orig.tar.xz: calligra-2.8.1/krita/sketch/MouseTracker.cpp <==
/* This file is part of the KDE project
 * Copyright (C) 2012 Arjen Hiemstra <ahiemstra@heimr.nl>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#include "MouseTracker.h"

#include <QDeclarativeItem>
#include <QEvent>
#include <QCoreApplication>
#include <QGraphicsSceneMouseEvent>
#include <QTouchEvent>

MouseTracker::MouseTracker(QObject* parent)
    : QObject(parent)
{
    QCoreApplication::instance()->installEventFilter(this);
}

MouseTracker::~MouseTracker()
{

}

void MouseTracker::addItem(QDeclarativeItem* item)
{
    m_trackedItems.append(item);
}

void MouseTracker::removeItem(QDeclarativeItem* item)
{
    m_trackedItems.removeOne(item);
}

bool MouseTracker::eventFilter(QObject* target, QEvent* event)
{
    Q_UNUSED(target)
    if (m_trackedItems.count() > 0) {
        switch(event->type()) {
            case QEvent::GraphicsSceneMouseMove: {
                QGraphicsSceneMouseEvent* mevent = static_cast<QGraphicsSceneMouseEvent*>(event);
                foreach(QDeclarativeItem* item, m_trackedItems) {
                    item->setPos(mevent->scenePos());
                }
                return false;
            }
            case QEvent::TouchUpdate: {
                QTouchEvent* tevent = static_cast<QTouchEvent*>(event);
                QTouchEvent::TouchPoint primary;
                foreach(const QTouchEvent::TouchPoint& point, tevent->touchPoints()) {
                    if (point.isPrimary()) {
                        primary = point;
                        break;
                    }
                }
                foreach(QDeclarativeItem* item, m_trackedItems) {
                    item->setPos(primary.scenePos());
                }
                return false;
            }
            case QEvent::DragMove: {
                QDragMoveEvent* mevent = static_cast<QDragMoveEvent*>(event);
                foreach(QDeclarativeItem* item, m_trackedItems) {
                    item->setPos(mevent->pos());
                }
                return false;
            }
            default: ;
        }
    }
    return false;
}

==> calligra_2.9.7.orig.tar.xz: calligra-2.9.7/krita/sketch/MouseTracker.cpp <==
/* This file is part of the KDE project
 * Copyright (C) 2012 Arjen Hiemstra <ahiemstra@heimr.nl>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#include "MouseTracker.h"

#include <QDeclarativeItem>
#include <QEvent>
#include <QCoreApplication>
#include <QGraphicsSceneMouseEvent>
#include <QTouchEvent>

struct TrackedItem {
    TrackedItem(QDeclarativeItem* i, const QPointF& o) : item(i), offset(o) { }

    QDeclarativeItem* item;
    QPointF offset;
};

class MouseTracker::Private
{
public:
    Private()
    { }

    QList<TrackedItem> trackedItems;
};

MouseTracker::MouseTracker(QObject* parent)
    : QObject(parent), d(new Private)
{
    QCoreApplication::instance()->installEventFilter(this);
}

MouseTracker::~MouseTracker()
{
    delete d;
}

void MouseTracker::addItem(QDeclarativeItem* item, const QPointF& offset)
{
    d->trackedItems.append(TrackedItem(item, offset));
}

void MouseTracker::removeItem(QDeclarativeItem* item)
{
    for(int i = 0; i < d->trackedItems.length(); ++i) {
        if(d->trackedItems.at(i).item == item) {
            d->trackedItems.removeAt(i);
            break;
        }
    }
}

bool MouseTracker::eventFilter(QObject* target, QEvent* event)
{
    Q_UNUSED(target)
    if (d->trackedItems.count() > 0) {
        switch(event->type()) {
            case QEvent::GraphicsSceneMouseMove: {
                QGraphicsSceneMouseEvent* mevent = static_cast<QGraphicsSceneMouseEvent*>(event);
                Q_FOREACH(const TrackedItem& item, d->trackedItems) {
                    item.item->setPos(mevent->scenePos() + item.offset);
                }
                return false;
            }
            case QEvent::TouchUpdate: {
                QTouchEvent* tevent = static_cast<QTouchEvent*>(event);
                QTouchEvent::TouchPoint primary;
                Q_FOREACH(const QTouchEvent::TouchPoint& point, tevent->touchPoints()) {
                    if (point.isPrimary()) {
                        primary = point;
                        break;
                    }
                }
                Q_FOREACH(const TrackedItem& item, d->trackedItems) {
                    item.item->setPos(primary.scenePos() + item.offset);
                }
                return false;
            }
            case QEvent::DragMove: {
                QDragMoveEvent* mevent = static_cast<QDragMoveEvent*>(event);
                Q_FOREACH(const TrackedItem& item, d->trackedItems) {
                    item.item->setPos(mevent->pos() + item.offset);
                }
                return false;
            }
            default: ;
        }
    }
    return false;
}
